Attribute VB_Name = "Snippets"
'**** Helper functions ****

Public Function InitStandalone() As Boolean
    Dim pInit As IAoInitialize
    Dim ProductList, Product
    Dim eProduct As esriLicenseProductCode
    Dim licenseStatus As esriLicenseStatus
    Set pInit = New AoInitialize
    ProductList = Array(esriLicenseProductCodeArcEditor, esriLicenseProductCodeArcView)
    For Each Product In ProductList
        eProduct = Product
        licenseStatus = pInit.IsProductCodeAvailable(eProduct)
        If licenseStatus = esriLicenseAvailable Then
            licenseStatus = pInit.Initialize(eProduct)
            InitStandalone = (licenseStatus = esriLicenseCheckedOut)
            Exit Function
        End If
    Next
End Function

Public Function GetApp(Optional app As String = "ArcMap") As IApplication
    If Not (app = "ArcMap" Or app = "ArcCatalog") Then
        Debug.Print "app must be 'ArcMap' or 'ArcCatalog'"
        Exit Function
    End If
    Dim pAppROT As IAppROT
    Dim iCount As Long, i As Long
    Dim pApp As IApplication
    Dim pDoc As IDocument
    Set pAppROT = New AppROT
    iCount = pAppROT.Count
    If iCount = 0 Then
        Exit Function
    End If
    For i = 0 To iCount - 1
        Set pApp = pAppROT.Item(i)
        If app = "ArcCatalog" Then
            If TypeOf pApp Is IGxApplication Then
                Set GetApp = pApp
                Exit Function
            End If
        ElseIf TypeOf pApp Is IMxApplication Then
            Set GetApp = pApp
            Exit Function
        End If
    Next
End Function

'**** Standalone ****

Sub Standalone_OpenFileGDB()

    Dim bResult As Boolean
    
    bResult = InitStandalone()
    If Not bResult Then
        Debug.Print "We've got lumps of it 'round the back..."
        Exit Sub
    End If
    
    Dim sPath As String
    Dim pWSF As IWorkspaceFactory
    Dim pWS As IWorkspace
    Dim pDS As IDataset
    
    sPath = "c:\apps\Demo\Montgomery_full.gdb"
    Set pWSF = New FileGDBWorkspaceFactory
    Set pWS = pWSF.OpenFromFile(sPath, 0)
    Set pDS = pWS
    Debug.Print "Workspace name: " & pDS.BrowseName
    Debug.Print "Workspace category: " & pDS.Category

End Sub

Sub Standalone_OpenSDE()

    Dim bResult As Boolean
    
    bResult = InitStandalone()
    
    Dim pPropSet As IPropertySet
    Dim pWSF As IWorkspaceFactory
    Dim pWS As IWorkspace
    Dim pDS As IDataset
    
    Set pPropSet = New PropertySet
    With pPropSet
        .SetProperty "SERVER", "sunprod1"
        .SetProperty "USER", "/"
        .SetProperty "INSTANCE", "sde:oracle10g:/;LOCAL=PRODUCTION_TUCSON"
        .SetProperty "AUTHENTICATION_MODE", "OSA"
        .SetProperty "VERSION", "SDE.DEFAULT"
    End With
    Set pWSF = New SdeWorkspaceFactory
    Set pWS = pWSF.Open(pPropSet, 0)
    Set pDS = pWS
    Debug.Print "Workspace name: " & pDS.BrowseName
    Debug.Print "Workspace category: " & pDS.Category

End Sub

Sub Standalone_OpenGISServer()

    Dim pServerConn As IGISServerConnection
    Dim pServerManager As IServerObjectManager
    Dim pServerContext As IServerContext
    
    Set pServerConn = New GISServerConnection
    pServerConn.Connect "tuswpesri02"
    Set pServerManager = pServerConn.ServerObjectManager
    Set pServerContext = pServerManager.CreateServerContext("", "")

    Dim XList, YList
    Dim pPtColl As IPointCollection
    Dim pPoint As IPoint
    Dim i As Long
    Dim pArea As IArea
    
    XList = Array(0, 0, 10, 10)
    YList = Array(0, 10, 10, 0)
    Set pPtColl = pServerContext.CreateObject("esriGeometry.Polygon")
    For i = 0 To 3
        Set pPoint = pServerContext.CreateObject("esriGeometry.Point")
        pPoint.PutCoords XList(i), YList(i)
        pPtColl.AddPoint pPoint
    Next
    Set pArea = pPtColl
    Debug.Print "Area = " & pArea.Area
    
    pServerContext.ReleaseContext

End Sub

Sub Standalone_QueryDBValues()
    Dim bResult As Boolean
    
    bResult = InitStandalone()
    
    Dim sPath As String, sTabName As String, sWhereClause As String, sFieldName As String
    Dim pWSF As IWorkspaceFactory
    Dim pWS As IWorkspace
    Dim pFWS As IFeatureWorkspace
    Dim pTab As ITable
    Dim pQF As IQueryFilter
    Dim pCursor As ICursor
    Dim pRow As IRow
    Dim Val
    
    sPath = "c:\apps\Demo\Montgomery_full.gdb"
    sTabName = "Parcels"
    sWhereClause = "parcel_id = 6358"
    sFieldName = "zoning_s"
    
    Set pWSF = New FileGDBWorkspaceFactory
    Set pWS = pWSF.OpenFromFile(sPath, 0)
    Set pFWS = pWS
    Set pTab = pFWS.OpenTable(sTabName)
    Set pQF = New QueryFilter
    pQF.WhereClause = sWhereClause
    Set pCursor = pTab.Search(pQF, True)
    Set pRow = pCursor.NextRow
    If pRow Is Nothing Then
        Debug.Print "Query returned no rows"
        Exit Sub
    End If
    Val = pRow.Value(pTab.FindField(sFieldName))
    If IsNull(Val) Then
        Debug.Print "Null value"
    End If

End Sub

Sub Standalone_CreateTable()

    Dim bResult As Boolean
    
    bResult = InitStandalone()
    
    Dim sPath As String
    Dim pWSF As IWorkspaceFactory
    Dim pWS As IWorkspace
    Dim pFWS As IFeatureWorkspace
    
    sPath = "c:\apps\ARDemo\Temp.gdb"
    Set pWSF = New FileGDBWorkspaceFactory
    Set pWS = pWSF.OpenFromFile(sPath, 0)
    Set pFWS = pWS

    Dim pOutFields As IFields
    Dim pFieldsEdit As IFieldsEdit
    Dim pNewField As iField
    Dim pFieldEdit As IFieldEdit
    Dim pOutTable As ITable

    Set pOutFields = New esriGeoDatabase.Fields
    Set pFieldsEdit = pOutFields
    pFieldsEdit.FieldCount = 2
    Set pNewField = New esriGeoDatabase.Field
    Set pFieldEdit = pNewField
    With pFieldEdit
        .Name = "OBJECTID"
        .Type = esriFieldTypeOID
    End With
    Set pFieldsEdit.Field(0) = pNewField
    Set pNewField = New esriGeoDatabase.Field
    Set pFieldEdit = pNewField
    With pFieldEdit
        .Name = "LUMBERJACK"
        .Type = esriFieldTypeString
        .Length = 50
    End With
    Set pFieldsEdit.Field(1) = pNewField
    Set pOutTable = pFWS.CreateTable("Test", pOutFields, Nothing, Nothing, "")

    Dim iField As Long
    Dim pRow As IRow
    
    iField = pOutTable.FindField("LUMBERJACK")
    Debug.Print "'LUMBERJACK' field index = " & iField
    Set pRow = pOutTable.CreateRow
    pRow.Value(iField) = "I sleep all night and I work all day"
    pRow.Store

End Sub

'**** ArcMap ****

Public Sub ArcMap_GetSelectedGeometry()

    Dim pApp As IApplication
    
    Set pApp = GetApp()
    If pApp Is Nothing Then
        Debug.Print "We found this spoon, sir."
        Exit Sub
    End If
    
    ' Loop through layers in focus map
    
    Dim pDoc As IDocument
    Dim pMxDoc As IMxDocument
    Dim pMap As IMap
    Dim pFeatSel As IFeatureSelection
    Dim pEnumFeat As IEnumFeature
    Dim pFeat As IFeature
    Dim pShape As IGeometry
    Dim eType As esriGeometryType
    
    Set pDoc = pApp.Document
    Set pMxDoc = pDoc
    Set pMap = pMxDoc.FocusMap
    Set pFeatSel = pMap.FeatureSelection
    Set pEnumFeat = pFeatSel
    pEnumFeat.Reset
    Set pFeat = pEnumFeat.Next
    If pFeat Is Nothing Then
        Debug.Print "No selection found."
        Exit Sub
    End If
    Set pShape = pFeat.ShapeCopy
    eType = pShape.GeometryType
    If eType = esriGeometryPoint Then
        Debug.Print "Geometry type = Point"
    ElseIf eType = esriGeometryPolyline Then
        Debug.Print "Geometry type = Line"
    ElseIf eType = esriGeometryPolygon Then
        Debug.Print "Geometry type = Poly"
    Else
        Debug.Print "Geometry type = Other"
    End If

End Sub

Public Sub ArcMap_AddTextElement()

    Dim pApp As IApplication
    Dim pFact As IObjectFactory
    
    Set pApp = GetApp()
    Set pFact = pApp
    
    ' Get midpoint of focus map
    
    Dim pDoc As IDocument
    Dim pMxDoc As IMxDocument
    Dim pMap As IMap
    Dim pAV As IActiveView
    Dim pSD As IScreenDisplay
    Dim pEnv As IEnvelope
    Dim dX As Double, dY As Double
    Dim pPt As IPoint
    
    Set pDoc = pApp.Document
    Set pMxDoc = pDoc
    Set pMap = pMxDoc.FocusMap
    Set pAV = pMap
    Set pSD = pAV.ScreenDisplay
    Set pEnv = pAV.Extent
    dX = (pEnv.XMin + pEnv.XMax) / 2
    dY = (pEnv.YMin + pEnv.YMax) / 2
    Set pPt = pFact.Create("esriGeometry.Point")
    pPt.PutCoords dX, dY
    
    ' Create text symbol
    
    Dim pColor As IRgbColor
    Dim pFontDisp As IFontDisp
    Dim pTextSymbol As ITextSymbol
    
    Set pColor = pFact.Create("esriDisplay.RgbColor")
    pColor.Red = 255
    Set pFontDisp = pFact.Create("StdFont")
    pFontDisp.Name = "Arial"
    pFontDisp.Bold = True
    Set pTextSymbol = pFact.Create("esriDisplay.TextSymbol")
    pTextSymbol.Font = pFontDisp
    pTextSymbol.Color = pColor
    pTextSymbol.Size = 24
    
    ' Create text element and add it to map
    
    Dim pTextElement As ITextElement
    Dim pElement As IElement
    Dim pGC As IGraphicsContainer
    Dim pGCSel As IGraphicsContainerSelect
    Dim iOpt As Long
    
    Set pTextElement = pFact.Create("esriCarto.TextElement")
    pTextElement.Symbol = pTextSymbol
    pTextElement.Text = "Wink, wink, nudge, nudge," & vbNewLine & "Say no more!"
    Set pElement = pTextElement
    pElement.Geometry = pPt
    
    Set pGC = pMap
    pGC.AddElement pElement, 0
    Set pGCSel = pMap
    pGCSel.SelectElement pElement
    iOpt = esriViewGraphics + esriViewGraphicSelection
    pAV.PartialRefresh iOpt, Nothing, Nothing
    
    ' Get element width
    
    Dim iCount As Long
    
    iCount = pGCSel.ElementSelectionCount
    Set pElement = pGCSel.SelectedElement(iCount - 1)
    Set pEnv = New esriGeometry.Envelope
    pElement.QueryBounds pSD, pEnv
    Debug.Print "Width = " & pEnv.Width

End Sub

Sub ArcMap_GetEditWorkspace()

    Dim pApp As IApplication
    Dim pID As New UID
    Dim pEditor As IEditor
    Dim pWS As IWorkspace
    Dim pDS As IDataset

    Set pApp = GetApp()
    pID.Value = "esriEditor.Editor"
    Set pEditor = pApp.FindExtensionByCLSID(pID)
    If pEditor.EditState = esriStateEditing Then
        Set pWS = pEditor.EditWorkspace
        Set pDS = pWS
        Debug.Print "Workspace name: " & pDS.BrowseName
        Debug.Print "Workspace category: " & pDS.Category
    Else
        Debug.Print "No edit workspace"
    End If

End Sub

Sub ArcMap_GetSelectedTable()
    
    Dim pApp As IApplication
    Dim pDoc As IDocument
    Dim pMxDoc As IMxDocument
    Dim pUnk As IUnknown
    Dim pTable As ITable
    Dim pDS As IDataset

    Set pApp = GetApp()
    Set pDoc = pApp.Document
    Set pMxDoc = pDoc
    Set pUnk = pMxDoc.SelectedItem
    If pUnk Is Nothing Then
        Debug.Print "Nothing selected."
        Exit Sub
    End If
    If Not TypeOf pUnk Is ITable Then
        Debug.Print "No table selected."
        Exit Sub
    End If
    Set pTable = pUnk
    Set pDS = pTable
    Debug.Print "Selected table: " & pDS.Name
    
End Sub

'**** ArcCatalog ****

Sub ArcCatalog_GetSelectedTable()

    Dim pApp As IApplication
    Dim pGxApp As IGxApplication
    Dim pGxObj As IGxObject
    Dim pGxDS As IGxDataset
    Dim eType As esriDatasetType
    Dim pDS As IDataset
    Dim pTable As ITable

    Set pApp = GetApp("ArcCatalog")
    Set pGxApp = pApp
    Set pGxObj = pGxApp.SelectedObject
    If pGxObj Is Nothing Then
        Debug.Print "Nothing selected."
        Exit Sub
    End If
    If Not TypeOf pGxObj Is IGxDataset Then
        Debug.Print "No dataset selected."
        Exit Sub
    End If
    Set pGxDS = pGxObj
    eType = pGxDS.Type
    If Not (eType = esriDTFeatureClass Or eType = esriDTTable) Then
        Debug.Print "No table selected."
        Exit Sub
    End If
    Set pDS = pGxDS.Dataset
    Set pTable = pDS
    Debug.Print "Selected table: " & pDS.Name

End Sub

