@echo off

REM     Batch process to install dependencies for the SBAS module.
REM     Current dependencies:   requests, comtypes

echo Searching registry settings for ArcGIS...

setlocal ENABLEEXTENSIONS
setlocal EnableDelayedExpansion
set KEY_NAME="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\ESRI"

rem Get python version number from registry
FOR /F "usebackq" %%A IN (`REG QUERY %KEY_NAME% /F Python 2^>nul`) DO (
    set KEY=%%A
    if "!KEY:~0,4!"=="HKEY" (
        set PYVERSION=!KEY:~45!
        set REGKEY=!KEY!
        set ARCVERSION=!KEY:~51!
        echo Arc Version: !ARCVERSION!
    )
)

rem Get python location from the registry
FOR /F "usebackq skip=2 tokens=1-3" %%A IN (`REG QUERY %REGKEY% /V PythonDir 2^>nul`) DO (
    set PYTHONDIR=%%CArcGIS%ARCVERSION%
)

set python=%PYTHONDIR%\python.exe

if not exist %python% (
    rem     Fall back to the old hard-coded logic in the hope
    rem     that we can find Python that way.

    set pypath=%HOMEDRIVE%\Python27\ArcGIS10.

    if exist %pypath%3 (
        set python=%pypath%3\python.exe
    ) else if exist %pypath%2 (
        set python=%pypath%2\python.exe
    ) else if exist %pypath%4 (
        set python=%pypath%4\python.exe
    ) else if exist %pypath%1 (
        set python=%pypath%1\python.exe
    ) else (
        echo Unknown Python install path.
        pause
        exit 1
    )
)

echo Python Location: %python%

echo Beginning install momentarily...
timeout 4

cd comtypes-0.6.2
%python% setup.py build install clean
cd ..\kennethreitz-requests-673bd6a
%python% setup.py build install clean
pause